% Script: TextConv2.m (v. 2.0)
%
% This script demonstates the use of inputdlg, textscan and related
% functionality. It prompts the user for a file name and text to find
% and replace. The use of header lines in data files is also demonstrated.
% 
% Last revised 11/29/11 by Dr. C. S. Tritt

clc; clear;  % Clear command window and workspace. Be careful with 'old' 
%     cell arrays.
fclose('all'); % Close any open files.

% Get input & output base file name & build file names.
cFName = inputdlg('Enter base name of file to read & write:', ...
   'Base Name');
finName = [char(cFName) '.txt'];
foutName = [char(cFName) '.out'];

fin = fopen(finName, 'r'); % Open the input file for reading.

% Read from the input file using textscan. Note that result is a 1x1 cell 
% array containing a Nx1 cell array of strings (with each word in its own 
% cell). 
rawText = textscan(fin, '%s', 'HeaderLines', 1);

fclose(fin); % Close the input file when done with it.

% Get text to find and replace. Braces force cellstr array which inputdlg
% expects.
frText = inputdlg({'Find:'; 'Replace:'}, 'Find & Replace');
% At this point frText{1} refers to a string and frText(1) refers to a 1x1
% cell array containing a string.

% I would perfer to use vectorized (no explicit loop) code here, but I 
% couldn't get it to work when there was more than one match! Preallocate
% newText for efficiency. Loop over all input words, replacing matches.
newText = cell(size(rawText{1}));
for n = 1:size(rawText{1}, 1)
   if strcmp(frText{1}, rawText{1}(n))
      newText{n} = char(frText{2}); % Replace matching word.
   else
      newText{n} = char(rawText{1}(n)); % Copy non-matching word.
   end
end

% Use sprintf to write output text as a single large a string. Note that
% sprintf accepts a cell array of strings, but not a cell array of
% cellstrs. This results in the char conversion functions being needed 
% above.
outText = sprintf('%s ', newText{:}); % Reuse the %s specifier as needed.
outText = strtrim(outText); % Trim trailing space.

% Open output file for writting.
fout = fopen(foutName, 'w');

% Write a header line and the output (modified) text.
fprintf(fout, 'File: %s, Produced: %s, By textConv2.m\n', foutName, date); 
fprintf(fout, '%s', outText);

% Close the output file.
fclose(fout);